@@ -1,29 +1,28 @@
-""" Multicast DNS Service Discovery for Python, v0.14-wmcbrine
-    Copyright 2003 Paul Scott-Murphy, 2014 William McBrine
+"""Multicast DNS Service Discovery for Python, v0.14-wmcbrine
+Copyright 2003 Paul Scott-Murphy, 2014 William McBrine
 
-    This module provides a framework for the use of DNS Service Discovery
-    using IP multicast.
+This module provides a framework for the use of DNS Service Discovery
+using IP multicast.
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-    Lesser General Public License for more details.
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
-    USA
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
+USA
 """
 
 import asyncio
 import random
 import sys
-from ipaddress import IPv4Address, IPv6Address, _BaseAddress
 from typing import TYPE_CHECKING, Dict, List, Optional, Set, Union, cast
 
 from .._cache import DNSCache
@@ -50,6 +49,8 @@ from .._utils.asyncio import (
     wait_for_future_set_or_timeout,
 )
 from .._utils.ipaddress import (
+    ZeroconfIPv4Address,
+    ZeroconfIPv6Address,
     cached_ip_addresses,
     get_ip_address_object_from_record,
     ip_bytes_and_scope_to_address,
@@ -137,28 +138,28 @@ class ServiceInfo(RecordUpdateListener):
     """
 
     __slots__ = (
-        "text",
-        "type",
-        "_name",
-        "key",
-        "_ipv4_addresses",
-        "_ipv6_addresses",
-        "port",
-        "weight",
-        "priority",
-        "server",
-        "server_key",
-        "_properties",
         "_decoded_properties",
-        "host_ttl",
-        "other_ttl",
-        "interface_index",
-        "_new_records_futures",
+        "_dns_address_cache",
         "_dns_pointer_cache",
         "_dns_service_cache",
         "_dns_text_cache",
-        "_dns_address_cache",
         "_get_address_and_nsec_records_cache",
+        "_ipv4_addresses",
+        "_ipv6_addresses",
+        "_name",
+        "_new_records_futures",
+        "_properties",
+        "host_ttl",
+        "interface_index",
+        "key",
+        "other_ttl",
+        "port",
+        "priority",
+        "server",
+        "server_key",
+        "text",
+        "type",
+        "weight",
     )
 
     def __init__(
@@ -168,7 +169,7 @@ class ServiceInfo(RecordUpdateListener):
         port: Optional[int] = None,
         weight: int = 0,
         priority: int = 0,
-        properties: Union[bytes, Dict] = b'',
+        properties: Union[bytes, Dict] = b"",
         server: Optional[str] = None,
         host_ttl: int = _DNS_HOST_TTL,
         other_ttl: int = _DNS_OTHER_TTL,
@@ -183,12 +184,12 @@ class ServiceInfo(RecordUpdateListener):
         if not type_.endswith(service_type_name(name, strict=False)):
             raise BadTypeInNameException
         self.interface_index = interface_index
-        self.text = b''
+        self.text = b""
         self.type = type_
         self._name = name
         self.key = name.lower()
-        self._ipv4_addresses: List[IPv4Address] = []
-        self._ipv6_addresses: List[IPv6Address] = []
+        self._ipv4_addresses: List[ZeroconfIPv4Address] = []
+        self._ipv6_addresses: List[ZeroconfIPv6Address] = []
         if addresses is not None:
             self.addresses = addresses
         elif parsed_addresses is not None:
@@ -220,7 +221,7 @@ class ServiceInfo(RecordUpdateListener):
 
     @name.setter
     def name(self, name: str) -> None:
-        """Replace the the name and reset the key."""
+        """Replace the name and reset the key."""
         self._name = name
         self.key = name.lower()
         self._dns_service_cache = None
@@ -260,11 +261,11 @@ class ServiceInfo(RecordUpdateListener):
                 )
             if addr.version == 4:
                 if TYPE_CHECKING:
-                    assert isinstance(addr, IPv4Address)
+                    assert isinstance(addr, ZeroconfIPv4Address)
                 self._ipv4_addresses.append(addr)
             else:
                 if TYPE_CHECKING:
-                    assert isinstance(addr, IPv6Address)
+                    assert isinstance(addr, ZeroconfIPv6Address)
                 self._ipv6_addresses.append(addr)
 
     @property
@@ -321,7 +322,7 @@ class ServiceInfo(RecordUpdateListener):
 
     def ip_addresses_by_version(
         self, version: IPVersion
-    ) -> Union[List[IPv4Address], List[IPv6Address], List[_BaseAddress]]:
+    ) -> Union[List[ZeroconfIPv4Address], List[ZeroconfIPv6Address]]:
         """List ip_address objects matching IP version.
 
         Addresses are guaranteed to be returned in LIFO (last in, first out)
@@ -334,7 +335,7 @@ class ServiceInfo(RecordUpdateListener):
 
     def _ip_addresses_by_version_value(
         self, version_value: int_
-    ) -> Union[List[IPv4Address], List[IPv6Address]]:
+    ) -> Union[List[ZeroconfIPv4Address], List[ZeroconfIPv6Address]]:
         """Backend for addresses_by_version that uses the raw value."""
         if version_value == _IPVersion_All_value:
             return [*self._ipv4_addresses, *self._ipv6_addresses]  # type: ignore[return-value]
@@ -369,21 +370,21 @@ class ServiceInfo(RecordUpdateListener):
         """Sets properties and text of this info from a dictionary"""
         list_: List[bytes] = []
         properties_contain_str = False
-        result = b''
+        result = b""
         for key, value in properties.items():
             if isinstance(key, str):
-                key = key.encode('utf-8')
+                key = key.encode("utf-8")
                 properties_contain_str = True
 
             record = key
             if value is not None:
                 if not isinstance(value, bytes):
-                    value = str(value).encode('utf-8')
+                    value = str(value).encode("utf-8")
                     properties_contain_str = True
-                record += b'=' + value
+                record += b"=" + value
             list_.append(record)
         for item in list_:
-            result = b''.join((result, bytes((len(item),)), item))
+            result = b"".join((result, bytes((len(item),)), item))
         if not properties_contain_str:
             # If there are no str keys or values, we can use the properties
             # as-is, without decoding them, otherwise calling
@@ -426,7 +427,7 @@ class ServiceInfo(RecordUpdateListener):
             length = text[index]
             index += 1
             key_value = text[index : index + length]
-            key_sep_value = key_value.partition(b'=')
+            key_sep_value = key_value.partition(b"=")
             key = key_sep_value[0]
             if key not in properties:
                 properties[key] = key_sep_value[2] or None
@@ -439,10 +440,10 @@ class ServiceInfo(RecordUpdateListener):
         return self._name[: len(self._name) - len(self.type) - 1]
 
     def _get_ip_addresses_from_cache_lifo(
-        self, zc: 'Zeroconf', now: float_, type: int_
-    ) -> List[Union[IPv4Address, IPv6Address]]:
+        self, zc: "Zeroconf", now: float_, type: int_
+    ) -> List[Union[ZeroconfIPv4Address, ZeroconfIPv6Address]]:
         """Set IPv6 addresses from the cache."""
-        address_list: List[Union[IPv4Address, IPv6Address]] = []
+        address_list: List[Union[ZeroconfIPv4Address, ZeroconfIPv6Address]] = []
         for record in self._get_address_records_from_cache_by_type(zc, type):
             if record.is_expired(now):
                 continue
@@ -452,25 +453,27 @@ class ServiceInfo(RecordUpdateListener):
         address_list.reverse()  # Reverse to get LIFO order
         return address_list
 
-    def _set_ipv6_addresses_from_cache(self, zc: 'Zeroconf', now: float_) -> None:
+    def _set_ipv6_addresses_from_cache(self, zc: "Zeroconf", now: float_) -> None:
         """Set IPv6 addresses from the cache."""
         if TYPE_CHECKING:
             self._ipv6_addresses = cast(
-                "List[IPv6Address]", self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_AAAA)
+                "List[ZeroconfIPv6Address]",
+                self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_AAAA),
             )
         else:
             self._ipv6_addresses = self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_AAAA)
 
-    def _set_ipv4_addresses_from_cache(self, zc: 'Zeroconf', now: float_) -> None:
+    def _set_ipv4_addresses_from_cache(self, zc: "Zeroconf", now: float_) -> None:
         """Set IPv4 addresses from the cache."""
         if TYPE_CHECKING:
             self._ipv4_addresses = cast(
-                "List[IPv4Address]", self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_A)
+                "List[ZeroconfIPv4Address]",
+                self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_A),
             )
         else:
             self._ipv4_addresses = self._get_ip_addresses_from_cache_lifo(zc, now, _TYPE_A)
 
-    def async_update_records(self, zc: 'Zeroconf', now: float_, records: List[RecordUpdate]) -> None:
+    def async_update_records(self, zc: "Zeroconf", now: float_, records: List[RecordUpdate]) -> None:
         """Updates service information from a DNS record.
 
         This method will be run in the event loop.
@@ -482,7 +485,7 @@ class ServiceInfo(RecordUpdateListener):
         if updated and new_records_futures:
             _resolve_all_futures_to_none(new_records_futures)
 
-    def _process_record_threadsafe(self, zc: 'Zeroconf', record: DNSRecord, now: float_) -> bool:
+    def _process_record_threadsafe(self, zc: "Zeroconf", record: DNSRecord, now: float_) -> bool:
         """Thread safe record updating.
 
         Returns True if a new record was added.
@@ -507,24 +510,32 @@ class ServiceInfo(RecordUpdateListener):
 
             if ip_addr.version == 4:
                 if TYPE_CHECKING:
-                    assert isinstance(ip_addr, IPv4Address)
+                    assert isinstance(ip_addr, ZeroconfIPv4Address)
                 ipv4_addresses = self._ipv4_addresses
                 if ip_addr not in ipv4_addresses:
                     ipv4_addresses.insert(0, ip_addr)
                     return True
-                elif ip_addr != ipv4_addresses[0]:
+                # Use int() to compare the addresses as integers
+                # since by default IPv4Address.__eq__ compares the
+                # the addresses on version and int which more than
+                # we need here since we know the version is 4.
+                elif ip_addr.zc_integer != ipv4_addresses[0].zc_integer:
                     ipv4_addresses.remove(ip_addr)
                     ipv4_addresses.insert(0, ip_addr)
 
                 return False
 
             if TYPE_CHECKING:
-                assert isinstance(ip_addr, IPv6Address)
+                assert isinstance(ip_addr, ZeroconfIPv6Address)
             ipv6_addresses = self._ipv6_addresses
             if ip_addr not in self._ipv6_addresses:
                 ipv6_addresses.insert(0, ip_addr)
                 return True
-            elif ip_addr != self._ipv6_addresses[0]:
+            # Use int() to compare the addresses as integers
+            # since by default IPv6Address.__eq__ compares the
+            # the addresses on version and int which more than
+            # we need here since we know the version is 6.
+            elif ip_addr.zc_integer != self._ipv6_addresses[0].zc_integer:
                 ipv6_addresses.remove(ip_addr)
                 ipv6_addresses.insert(0, ip_addr)
 
@@ -701,13 +712,16 @@ class ServiceInfo(RecordUpdateListener):
             self._get_address_and_nsec_records_cache = records
         return records
 
-    def _get_address_records_from_cache_by_type(self, zc: 'Zeroconf', _type: int_) -> List[DNSAddress]:
+    def _get_address_records_from_cache_by_type(self, zc: "Zeroconf", _type: int_) -> List[DNSAddress]:
         """Get the addresses from the cache."""
         if self.server_key is None:
             return []
         cache = zc.cache
         if TYPE_CHECKING:
-            records = cast("List[DNSAddress]", cache.get_all_by_details(self.server_key, _type, _CLASS_IN))
+            records = cast(
+                "List[DNSAddress]",
+                cache.get_all_by_details(self.server_key, _type, _CLASS_IN),
+            )
         else:
             records = cache.get_all_by_details(self.server_key, _type, _CLASS_IN)
         return records
@@ -721,14 +735,14 @@ class ServiceInfo(RecordUpdateListener):
             self.server = self._name
             self.server_key = self.key
 
-    def load_from_cache(self, zc: 'Zeroconf', now: Optional[float_] = None) -> bool:
+    def load_from_cache(self, zc: "Zeroconf", now: Optional[float_] = None) -> bool:
         """Populate the service info from the cache.
 
         This method is designed to be threadsafe.
         """
         return self._load_from_cache(zc, now or current_time_millis())
 
-    def _load_from_cache(self, zc: 'Zeroconf', now: float_) -> bool:
+    def _load_from_cache(self, zc: "Zeroconf", now: float_) -> bool:
         """Populate the service info from the cache.
 
         This method is designed to be threadsafe.
@@ -758,7 +772,7 @@ class ServiceInfo(RecordUpdateListener):
 
     def request(
         self,
-        zc: 'Zeroconf',
+        zc: "Zeroconf",
         timeout: float,
         question_type: Optional[DNSQuestionType] = None,
         addr: Optional[str] = None,
@@ -777,12 +791,15 @@ class ServiceInfo(RecordUpdateListener):
         :param addr: address to send the request to
         :param port: port to send the request to
         """
-        assert zc.loop is not None and zc.loop.is_running()
+        assert zc.loop is not None, "Zeroconf instance must have a loop, was it not started?"
+        assert zc.loop.is_running(), "Zeroconf instance loop must be running, was it already stopped?"
         if zc.loop == get_running_loop():
             raise RuntimeError("Use AsyncServiceInfo.async_request from the event loop")
         return bool(
             run_coro_with_timeout(
-                self.async_request(zc, timeout, question_type, addr, port), zc.loop, timeout
+                self.async_request(zc, timeout, question_type, addr, port),
+                zc.loop,
+                timeout,
             )
         )
 
@@ -794,7 +811,7 @@ class ServiceInfo(RecordUpdateListener):
 
     async def async_request(
         self,
-        zc: 'Zeroconf',
+        zc: "Zeroconf",
         timeout: float,
         question_type: Optional[DNSQuestionType] = None,
         addr: Optional[str] = None,
@@ -894,7 +911,7 @@ class ServiceInfo(RecordUpdateListener):
             out.add_answer_at_time(answer, now)
 
     def _generate_request_query(
-        self, zc: 'Zeroconf', now: float_, question_type: DNSQuestionType
+        self, zc: "Zeroconf", now: float_, question_type: DNSQuestionType
     ) -> DNSOutgoing:
         """Generate the request query."""
         out = DNSOutgoing(_FLAGS_QR_QUERY)
@@ -919,20 +936,20 @@ class ServiceInfo(RecordUpdateListener):
 
     def __repr__(self) -> str:
         """String representation"""
-        return '{}({})'.format(
+        return "{}({})".format(
             type(self).__name__,
-            ', '.join(
-                f'{name}={getattr(self, name)!r}'
+            ", ".join(
+                f"{name}={getattr(self, name)!r}"
                 for name in (
-                    'type',
-                    'name',
-                    'addresses',
-                    'port',
-                    'weight',
-                    'priority',
-                    'server',
-                    'properties',
-                    'interface_index',
+                    "type",
+                    "name",
+                    "addresses",
+                    "port",
+                    "weight",
+                    "priority",
+                    "server",
+                    "properties",
+                    "interface_index",
                 )
             ),
         )
